function result = flow(cmd, arg)
% DIVERGENCE
%
% This module helps you estimate the divergence of a vector field at
% a point.
% 
% Specify a vector field by entering the component functions, in
% terms of x and y, in the u(x,y) and v(x,y) boxes. Specify the
% coordinates of the point at which you wish to investigate the
% divergence. Pick a small "delta t" and a short side length.
% Interpret the vector field as a velocity field; the vector at
% (x,y) represents the velocity of a particle of fluid situated at
% (x,y).
% 
% When you press the FLOW button, four blue crosses and a red
% quadrilateral appear in the right of the window.  The four blue
% crosses mark the corners of a square centered at the specified
% point with the specified side length. The corners of the red
% quadrilateral represent the positions, after delta t seconds of
% flow, of particles that were originally at the blue crosses. The
% red quadrilateral gives you an idea of how the shape of a square
% portion of the fluid has changed after delta t seconds of flow.
% The approximate area of the red quadrilateral appears at the top
% of the window.
% 
% THIS MODULE WILL ONLY GIVE YOU ACCURATE PICTURES FOR FAIRLY SMALL
% VALUES OF DELTA T. This is because the corners of the red
% quadrilateral do not mark the actual positions of particles that
% flowed away from the blue crosses, but only the positions they
% would have if their velocities do not change much as they flow. 
% If delta t is small, the particles do not flow far, so their
% velocities do not change much, and you get more accurate picture.
%
% 2.0/16-July-1998
	
% HISTORY:
% 1.0 -- Original by Alex Gottlieb (Summer 1996)
% 2.0 -- "Modernized" by Tom Insel (February 1998)
%        added vector field in the bkgnd
%
% REQUIRES: cleanup.m
%
% TODO/BUGS:
% - Should have better error detection.

persistent u_of_xy v_of_xy x_naught y_naught delta_t delta_length red_text blue_text

% colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_vector = [.5 .5 .5];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	control_figures = findobj('tag', 'flow_main');
	if length(control_figures) == 0
		control_figures = flow('makemain');
	end

	figure(control_figures(1));

elseif strcmp(cmd, 'makemain')

	control_figure = figure(...
		'position', [10 10 600 400], ...
		'name', 'Divergence', ...
		'numbertitle', 'off',...
		'resize', 'off',...
		'color', math_bkgnd,...
		'Tag', 'flow_main');
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 340 180 1]);

	blue_text = uicontrol(control_figure,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'foregroundcolor', 'b',...
		'position',[15 365 170 20]);
	
	red_text = uicontrol(control_figure,...
		'style', 'text',...
		'backgroundcolor',math_bkgnd,...
		'foregroundcolor', 'r',...
		'position',[15 345 170 20]);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 310 180 20],...
		'backgroundcolor', math_bkgnd,...
		'string', 'FLUID VELOCITY VECTOR FIELD',...
		'horizontalalignment', 'left');

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [20 290 50 20],...
		'backgroundcolor', math_bkgnd,...
		'string', 'u(x,y) = ',...
		'horizontalalignment', 'right');

	u_of_xy = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [70 290 120 20],...
		'backgroundcolor', math_field,...
		'string', '5*x + y');

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [20 265 50 20],...
		'backgroundcolor', math_bkgnd,...
		'string', 'v(x,y) = ',...
		'horizontalalignment', 'right');

	v_of_xy = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [70 265 120 20],...
		'backgroundcolor', math_field,...
		'string', 'x + 5*y');

	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 255 180 1],...
		'backgroundcolor', 'k');

	uicontrol(control_figure,...
		'style', 'text',...
		'pos', [10 230 180 20],...
		'string', 'TIME INCREMENT',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
		
	uicontrol(control_figure,...
		'style', 'text',...
		'pos', [20 210 60 20],...
		'string', 'delta t',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	delta_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [90 210 100 20],...
		'string', '.01',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 200 180 1]);

	uicontrol(control_figure,...
		'style', 'text',...
		'pos', [10 175 180 20],...
		'string', 'SMALL SQUARE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
	
	uicontrol(control_figure,...
		'style', 'text',...
		'pos', [20 155 60 20],...
		'string', 'center point',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
			
	x_naught = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [90 155 50 20],...
		'string', '0',...
		'backgroundcolor', math_field);
	
	y_naught = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [140 155 50 20],...
		'string', '0',...
		'backgroundcolor', math_field);
	
	uicontrol(control_figure,...
		'style', 'text',...
		'pos', [10 130 70 20],...
		'string', 'side length',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	delta_length = uicontrol(control_figure,...
		'style', 'edit',...
		'pos', [90 130 100 20],...
		'string', '.1',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 120 180 1]);

	uicontrol(control_figure,...
		'position', [10 80 180 30],...
		'string', 'Flow',...
		'callback','flow(''doflow'');');
		
	uicontrol(control_figure,...
		'position', [10 45 180 30],...
		'string', 'Help',...
		'callback','helpwin flow');
		
	uicontrol(control_figure,...
		'position', [10 10 180 30],...
		'string', 'Quit',...
		'callback','flow(''end'');');

	flow_axes = axes('position',[.4 .1 .55 .8]);
	xlabel('x');
	ylabel('y');

	result = control_figure;

elseif strcmp(cmd, 'doflow')

	figures = findobj('tag', 'flow_main');
	if length(figures) == 0
		figures = flow('makemain');
	end

	main_figure = figures(1);

	u_string = cleanup( get( u_of_xy, 'string'));
	v_string = cleanup( get( v_of_xy, 'string'));
	x0 = str2num( cleanup( get( x_naught, 'string')));
	y0 = str2num( cleanup( get( y_naught, 'string')));
	dt = str2num( cleanup( get( delta_t, 'string')));
	dl = str2num( cleanup( get( delta_length, 'string')));

	figure(main_figure); 

	cla,
	axis([x0-dl x0+dl y0-dl y0+dl]) % should compute the desired size
	axis('square')
	hold on
	
	grid_resolution = 15;
	scale = 3;

	[x y] = meshgrid(...
		linspace(x0-dl,x0+dl, grid_resolution),...
		linspace(y0-dl,y0+dl, grid_resolution));
	P = eval(u_string);
	Q = eval(v_string);
	if length(P) == 1, P = P + 0*x; end
	if length(Q) == 1, Q = Q + 0*x; end
	q = quiver(x,y,P,Q,scale);
	set(q,'color', math_vector);

	a = [x0 + dl/2, x0 - dl/2, x0 - dl/2, x0 + dl/2];
	b = [y0 + dl/2, y0 + dl/2, y0 - dl/2, y0 - dl/2];
	plot(a,b,'b+');
	x = x0 + dl/2; y = y0 + dl/2;
	n1 = [x, y] + dt*[eval(u_string), eval(v_string)];
	x = x0 - dl/2; y = y0 + dl/2;
	n2 = [x, y] + dt*[eval(u_string), eval(v_string)];
	x = x0 - dl/2; y = y0 - dl/2;
	n3 = [x, y] + dt*[eval(u_string), eval(v_string)];
	x = x0 + dl/2; y = y0 - dl/2;
	n4 = [x, y] + dt*[eval(u_string), eval(v_string)];
	
	plot([n1(1) n2(1) n3(1) n4(1) n1(1)], [n1(2) n2(2) n3(2) n4(2) n1(2)], 'r');
		
	approx_area = det([n1-n2;n1-n4]);

	set(blue_text, 'string', [ 'Starting Area = ' num2str(dl*dl) ]);
	set(red_text, 'string', [ 'New Area  ' num2str(approx_area) ]);

	hold off

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'flow_main');	
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end
