% FOURIER SERIES
% This module is used to plot partial Fourier approximations.
%
% INSTRUCTIONS
% 1. Enter a piecewise-defined function f(x) by entering
%    it's definition on up to three pieces.   The program will
%    automatically adjust the limits so they make sense. 
%    To ignore a piece, make the high and low range equal.
% 2. Next adjust the slider to compute n terms and then click 
%    the 'Update Graph' button.  A large number of integrals 
%    will be computed and an approximation  to the fourier 
%    expansion will be plotted.  Be patient, this takes time.
% 3. Viewing Fourier Coefficients:
%    Adjust the slider bar on the right hand side to view the 
%    mth Fourier coefficient.  If the 'Compute Symbolically'
%    checkbox is selected then the coefficients will be computed
%    symbolically, otherwise they are computed numerically. 
% 4. Click on the examples button to see some standard examples.
%
% Author: William Stein, July 1998

function FourierSeries (action)

persistent h_Fig h_mText h_mSlider h_AmText h_BmText ...
	h_NumTermsSlider h_NumTermsText h_Axes ...
   h_SymbolicCheck res 
   
global h_L1Function h_L2Function h_L3Function ...
      h_L1Lower h_L1Upper h_L2Lower h_L2Upper h_L3Lower h_L3Upper ...
   	known_am value_am known_bm value_bm ...
   	known_amsym value_amsym known_bmsym value_bmsym 
   
if nargin == 0
   action = 'Init' ;
end

switch action
   
case 'Init',
   h_Fig = findobj ('Tag', 'FourierSeriesTool') ;
   if isempty (h_Fig)
      r = CreateFourierSeriesWindow ;
      h_Fig = r(1) ;
      h_L1Function = r(2) ;
      h_L2Function = r(3) ;
      h_L3Function = r(4) ;
      h_L1Lower = r(5) ;
      h_L1Upper = r(6) ;
      h_L2Lower = r(7) ;
      h_L2Upper = r(8) ;
      h_L3Lower = r(9) ;
      h_L3Upper = r(10) ;
      h_mText = r(11) ;
      h_mSlider = r(12) ;
      h_AmText = r(13) ;
      h_BmText = r(14) ;
      h_SymbolicCheck = r(15) ;
      h_NumTermsSlider = r(16) ;
      h_NumTermsText = r(17) ;
      h_Axes = r(18) ;
      known_am = [] ;
      known_bm = [] ;
      known_amsym = [] ;
      known_bmsym = [] ;
      res = 250 ;     % resolution
      FourierSeries PlotNow
   else
      figure (h_Fig) ;
   end
      
case 'L1Lower',
   try 
      ResetKnown
      L1Lower = str2num (cleanup (get (h_L1Lower, 'String'))) ;
      if L1Lower > 0 
         L1Lower = 0 ;
      end
      set (h_L1Lower, 'String', num2str(L1Lower)) ;
      set (h_L3Upper, 'String', num2str(-L1Lower)) ;
      MakeIntervalsValid
   catch
      errordlg (['There is an error in the lower limit of ' ...
            'the first interval of f(x).'], 'Input Error') 
   end
         
case 'L1Upper', 
   ResetKnown
   set (h_L2Lower, 'String', get(h_L1Upper, 'String')) ;
   MakeIntervalsValid
      
case 'L2Lower',
   ResetKnown
   set (h_L1Upper, 'String', get(h_L2Lower, 'String')) ;
   MakeIntervalsValid
   
case 'L2Upper',
   ResetKnown
   set (h_L3Lower, 'String', get(h_L2Upper, 'String')) ;
   MakeIntervalsValid
   
case 'L3Lower',   
   ResetKnown
   set (h_L2Upper, 'String', get(h_L3Lower, 'String')) ;
   MakeIntervalsValid
   
case 'L3Upper',
   try
      ResetKnown
      L3Upper = str2num (cleanup (get (h_L3Upper, 'String'))) ;
   	if L3Upper < 0 
      	L3Upper = 0 ;
	   end
   	set (h_L1Lower, 'String', num2str(-L3Upper)) ;
      set (h_L3Upper, 'String', num2str(L3Upper)) ;
      MakeIntervalsValid
   catch
      errordlg (['There is an error in the upper limit of ' ...
            'the last interval of f(x).'], 'Input Error') 
	end   
   
case 'L1Function',
   try
      ResetKnown ;
      L1Lower = str2num (cleanup (get (h_L1Lower, 'String'))) ;
   	L1Upper = str2num (cleanup (get (h_L1Upper, 'String'))) ;
      F = cleanup (get (h_L1Function, 'String')) ;
      x = linspace (L1Lower, L1Upper, res) ;
      dummy = eval ( vectorize (F) ) ;
	   FourierSeries ClearFourierCoeff
   catch
      errordlg (['There is an error in the definition of f(x)' ...
            ' on the first interval.'], 'Input Error') ;
   end
   
case 'L2Function',
   try
      ResetKnown
      L2Lower = str2num (cleanup (get (h_L2Lower, 'String'))) ;
   	L2Upper = str2num (cleanup (get (h_L2Upper, 'String'))) ;
      F = cleanup (get (h_L2Function, 'String')) ;
      x = linspace (L2Lower, L2Upper, res) ;
      dummy = eval ( vectorize (F) ) ;
	   FourierSeries ClearFourierCoeff
   catch
      errordlg (['There is an error in the definition of f(x)' ...
            ' on the second interval.'], 'Input Error') ;
   end
   
case 'L3Function',
   try
      ResetKnown
      L3Lower = str2num (cleanup (get (h_L3Lower, 'String'))) ;
   	L3Upper = str2num (cleanup (get (h_L3Upper, 'String'))) ;
      F = cleanup (get (h_L3Function, 'String')) ;
      x = linspace (L3Lower, L3Upper, res) ;
      dummy = eval ( vectorize (F) ) ;
      FourierSeries ClearFourierCoeff
   catch
      errordlg (['There is an error in the definition of f(x)' ...
            ' on the third interval.'], 'Input Error') ;
   end
   
case 'UpdateFourierCoeff',
   m = round(get (h_mSlider, 'Value')) ;
   set (h_mText, 'String', num2str (m)) 
   dosym = get (h_SymbolicCheck, 'Value') ;
   set (h_AmText, 'String', Am (m,dosym)) 
   if m > 0
   	set (h_BmText, 'String', Bm (m,dosym)) 
   else
      set (h_BmText, 'String', '') 
   end
   
case 'ClearFourierCoeff',
   set (h_AmText, 'String', '') 
   set (h_BmText, 'String', '') 
         
case 'PlotNow',
   m = round (get (h_NumTermsSlider, 'Value')) ;
   try
      L1Lower = str2num (cleanup (get (h_L1Lower, 'String'))) ;
      L1Upper = str2num (cleanup (get (h_L1Upper, 'String'))) ;
      L2Lower = L1Upper ;
      L2Upper = str2num (cleanup (get (h_L2Upper, 'String'))) ;
      L3Lower = L2Upper ;
      L3Upper = -L1Lower ;  
      
      F1 = cleanup (get (h_L1Function, 'String')) ;
      F2 = cleanup (get (h_L2Function, 'String')) ;
      F3 = cleanup (get (h_L3Function, 'String')) ;
      
      L = L3Upper ;
      x = L1Lower : L/res : L1Upper ;         
      y = eval ( vectorize (F1) ) + zeros (1,length(x));
      hold off
      plot (x, y, '-b') ;
      hold on 
      plot (x+2*L,y,'-b') ;
      plot (x-2*L,y,'-b') ;
      
      x = L2Lower : L/res : L2Upper ;         
      y = eval ( vectorize (F2) ) + zeros (1,length(x)) ;
      plot (x, y, '-b') 
      plot (x+2*L,y,'-b') ;
      plot (x-2*L,y,'-b') ;
      
      x = L3Lower : L/res : L3Upper ;         
      y = eval ( vectorize (F3) ) + zeros (1,length(x));
      plot (x, y, '-b') ;
      plot (x+2*L,y,'-b') ;
      plot (x-2*L,y,'-b') ;

      x = linspace (3*L1Lower, 3*L3Upper, res) ;
      y = eval (vectorize (MthOrderFourier (m))) + zeros (1,length(x));
      h_Four = plot (x, y, 'r') ;         
      
      y=get(gca,'ylim');
      set(gca,'ylim',[y(1)-.25,y(2)+.25])
      
      xlabel ('a_0/2 + a_1 cos (\pi x/L) + b_1 sin(\pi x/L) + ... + a_k cos(2\pi x/L) + b_k sin(2\pi x/L) ') ;
      
  catch
      errordlg ('Error making graph.', 'Graphing Error') ;   
      error(lasterr) 
  end
   
case 'PlotSlider',
   m = round (get (h_NumTermsSlider, 'Value')) ;
   set (h_NumTermsText, 'String', num2str (m)) ;   
   
case 'Help',
   helpwin('FourierSeries') 
   
case 'Examples',
   e(1).button = 'Triangular Wave' ;
   e(1).text = 'An even triangular wave periodic with period 4.' ;
   d(1).data = {'-x','-2','0','x','0','2','0','2','2'} ;
   e(2).button = 'Step Function' ;
   e(2).text = 'A step function with period 6.' ;
   d(2).data = {'0','-3','-1','1','-1','1','0','1','3'};
   e(3).button = 'Sawtooth Wave' ;
   e(3).text = ['An example of an odd function.  ' ...
         'Note that the odd terms of the Fourier expansion vanish.'] ;
   d(3).data = {'x','-2','2','0','2','2','0','2','2'};
   i = examples('FOURIER SERIES', e) ;
   if i
      d = d(i).data ;
      set(h_L1Function,'string',d{1}) ;
      set(h_L1Lower,'string',d{2}) ;
      set(h_L1Upper,'string',d{3}) ;
      set(h_L2Function,'string',d{4}) ;
      set(h_L2Lower,'string',d{5}) ;
      set(h_L2Upper,'string',d{6}) ;
      set(h_L3Function,'string',d{7}) ;
      set(h_L3Lower,'string',d{8}) ;
      set(h_L3Upper,'string',d{9}) ;
	   ResetKnown
      
      FourierSeries ClearFourierCoeff
      FourierSeries PlotNow
   end
   
case 'Quit'
   delete (findobj ('tag', 'FourierSeriesTool')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
end


function MakeIntervalsValid

global h_L1Function h_L2Function h_L3Function h_L1Lower ...
   h_L1Upper h_L2Lower h_L2Upper h_L3Lower h_L3Upper 

try
   L1Lower = str2num (cleanup (get (h_L1Lower, 'String'))) ;
   if L1Lower > 0 
      L1Lower = 0 
   end
   L1Upper = str2num (cleanup (get (h_L1Upper, 'String'))) ;
   L1Upper = max (L1Upper, L1Lower) ;
   L1Upper = min (L1Upper, -L1Lower) ;
   set (h_L1Upper, 'String', num2str (L1Upper)) ;
   set (h_L2Lower, 'String', num2str (L1Upper)) ;
   L2Upper = str2num (cleanup (get (h_L2Upper, 'String'))) ;
   L2Upper = max (L1Upper, L2Upper) ;
   L2Upper = min (L2Upper, -L1Lower) ;
   set (h_L2Upper, 'String', num2str (L2Upper)) ;
   set (h_L3Lower, 'String', num2str (L2Upper)) ;
   set (h_L3Upper, 'String', num2str(-L1Lower)) ;
   FourierSeries ClearFourierCoeff
catch
   errordlg (['There is an error somewhere in the intervals' ...
         ' in the definition of f(x).'], 'Fourier Series Error') ;
end

% return the m-th Fourier coefficient a_m as a string
% or return the empty string '' if can't compute it
% or it doesn't make sense.
function AmText = Am (m, dosym)
if nargin == 1
   dosym = 0 ;
end

global h_L1Function h_L2Function h_L3Function ...
   h_L1Lower h_L1Upper h_L2Lower h_L2Upper h_L3Lower h_L3Upper ...
	known_am value_am known_bm value_bm ...
	known_amsym value_amsym known_bmsym value_bmsym ...
	IntFun

if dosym & m <= length (known_amsym) - 1 & known_amsym (m+1)  
   AmText = value_amsym {m+1} ;
elseif ~dosym & m <= length (known_am) - 1 &  known_am (m+1) 
   AmText = value_am {m+1} ;
else
   try
      L1Lower = str2num (cleanup (get (h_L1Lower, 'String'))) ;
      L1Upper = str2num (cleanup (get (h_L1Upper, 'String'))) ;
      L2Lower = L1Upper ;
      L2Upper = str2num (cleanup (get (h_L2Upper, 'String'))) ;
      L3Lower = L2Upper ;
      L3Upper = -L1Lower ;  
      L = num2str (L3Upper) ;
      
      F1 = ['(' cleanup(get (h_L1Function, 'String')) ...
            ')/' L ' * cos (' num2str(m) '* pi * x /' L ')'] ;
      F2 = ['(' cleanup(get (h_L2Function, 'String')) ...
            ')/' L ' * cos (' num2str(m) '* pi * x /' L ')'] ;
      F3 = ['(' cleanup(get (h_L3Function, 'String')) ...
            ')/' L ' * cos (' num2str(m) '* pi * x /' L ')'] ;
      
      if havesymbolic & dosym
         x=sym('x'); 
         am = int (F1, L1Lower, L1Upper) ...
            + int (F2, L2Lower, L2Upper) ...
            + int (F3, L3Lower, L3Upper) ;
         AmText = char (am) ;
         value_amsym {m+1} = AmText ;
         known_amsym (m+1) = 1 ;
      else
         
         IntFun = vectorize(F1) ;
         am = quad('Fourier_IntFun', L1Lower, L1Upper,1/3) ;
         IntFun = vectorize(F2) ;
         am = am + quad('Fourier_IntFun', L2Lower, L2Upper,1/3) ;
         IntFun = vectorize(F3) ;
         am = am + quad('Fourier_IntFun', L3Lower, L3Upper,1/3) ;
         
         AmText = num2str (am) ;
      	value_am {m+1} = AmText ;
 	     	known_am (m+1) = 1 ;
      end         
 
      
	catch
      errordlg (['Error integrating to find a_' num2str(m) '.'], ...
         'Integration Error') ;
   end
end      

% return the m-th fourier coefficient b_m as a string
% or return the empty string '' if can't compute it
% or it doesn't make sense.
function BmText = Bm (m, dosym) 

if nargin==1 
   dosym = 0 ;
end

global h_L1Function h_L2Function h_L3Function ...
   h_L1Lower h_L1Upper h_L2Lower h_L2Upper h_L3Lower h_L3Upper ...
	known_am value_am known_bm value_bm ...
	known_amsym value_amsym known_bmsym value_bmsym IntFun

if dosym & m <= length (known_bmsym) & known_bmsym (m)
   BmText = value_bmsym {m} ;
elseif ~dosym & m <= length (known_bm) & known_bm (m)
   BmText = value_bm {m} ;
else
   try
      L1Lower = str2num (cleanup (get (h_L1Lower, 'String'))) ;
      L1Upper = str2num (cleanup (get (h_L1Upper, 'String'))) ;
      L2Lower = L1Upper ;
      L2Upper = str2num (cleanup (get (h_L2Upper, 'String'))) ;
      L3Lower = L2Upper ;
      L3Upper = -L1Lower ;  
      L = num2str (L3Upper) ;
      
      F1 = ['(' cleanup(get (h_L1Function, 'String')) ...
            ')/' L ' * sin (' num2str(m) '* pi * x /' L ')'] ;
      F2 = ['(' cleanup(get (h_L2Function, 'String')) ...
            ')/' L ' * sin (' num2str(m) '* pi * x /' L ')'] ;
      F3 = ['(' cleanup(get (h_L3Function, 'String')) ...
            ')/' L ' * sin (' num2str(m) '* pi * x /' L ')'] ;
      
      if havesymbolic & dosym
      	x=sym('x');
      
         bm = int (F1, 'x', L1Lower, L1Upper) ...
           	+ int (F2, 'x', L2Lower, L2Upper) ...
            + int (F3, 'x', L3Lower, L3Upper) ;
			BmText = char (simple(simple(bm))) ;
         value_bmsym {m} = BmText ;
      	known_bmsym (m) = 1 ;
      else
         IntFun = vectorize(F1) ;
			bm = quad('Fourier_IntFun', L1Lower, L1Upper) ;
			IntFun = vectorize(F2) ;
			bm = bm + quad('Fourier_IntFun', L2Lower, L2Upper) ;
			IntFun = vectorize(F3) ;
			bm = bm + quad('Fourier_IntFun', L3Lower, L3Upper) ;
			BmText = num2str (bm) ;
         value_bm {m} = BmText ;
      	known_bm (m) = 1 ;
      end
         
   catch
      errordlg (['Error integrating to find b_' num2str(m) '.'], ...
         'Integration Error') ;
   end
end

function ResetKnown
global known_am known_bm known_amsym known_bmsym 
known_am = [] ;
known_bm = [] ;
known_amsym = [] ;
known_bmsym = [] ;

function r = MthOrderFourier (m)
r = [Am(0) '/2 + '] ;
for j=1:m
   r = [r '+' Am(j) ' * cos (' num2str(j) '*pi*x/L) ' ...
         ' + ' Bm(j) ' * sin (' num2str(j) '*pi*x/L) '] ;
end


function r = CreateFourierSeriesWindow()

col = [.6 .8 1] ;
ed = [1 1 1] ;

h_Fig = figure('Color',col, ...
	'MenuBar','none', ...
	'Name','FOURIER SERIES', ...
	'NumberTitle','off', ...
	'Position',[10 10 600 410], ...
	'Resize','off', ...
	'Tag','FourierSeriesTool');

h_Axes = axes('units', 'pixels', 'position', [40 45 530 210]) ;


uicontrol(h_Fig, ...
	'BackgroundColor',col , ...
   'Position',[160 380 80 20], ...
   'horizontalalignment','left',...
	'String','f(x) = ', ...
   'Style','text');
h_L1Function = uicontrol(h_Fig,...
   'backgroundcolor', ed, ...
	'Position',[160 360 80 20], ...
	'Callback','FourierSeries (''L1Function'')', ...
	'String','x', ...
	'Style','edit');
h_L1Lower = uicontrol(h_Fig, ...
   'backgroundcolor', ed, ...
	'Callback','FourierSeries (''L1Lower'')', ...
	'Position',[260 360 30 20], ...
	'String','-3', ...
   'Style','edit');
uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
	'Position',[290 360 30 20], ...
	'String','<x<', ...
	'Style','text');
h_L1Upper = uicontrol(h_Fig, ...
   'backgroundcolor', ed, ...
	'Callback','FourierSeries (''L1Upper'')', ...
	'Position',[320 360 30 20], ...
	'String','-1', ...
   'Style','edit');
h_L2Function = uicontrol(h_Fig,...
   'backgroundcolor', ed, ...
	'Callback','FourierSeries (''L2Function'')', ...
	'Position',[160 335 80 20], ...
	'String','-1', ...
   'Style','edit');
h_L2Lower = uicontrol(h_Fig, ...
	'BackgroundColor',[1 1 1], ...
	'Callback','FourierSeries (''L2Lower'')', ...
	'Position',[260 335 30 20], ...
	'String','-1', ...
   'Style','edit');
uicontrol(h_Fig, ...
   'BackgroundColor', col, ...
   'Position',[290 335 30 20], ...
	'String','<x<', ...
   'Style','text');
h_L2Upper = uicontrol(h_Fig, ...
	'BackgroundColor',[1 1 1], ...
	'Callback','FourierSeries (''L2Upper'')', ...
	'Position',[320 335 30 20], ...
	'String','1', ...
	'Style','edit');
h_L3Function = uicontrol(h_Fig, ...
   'backgroundcolor', ed, ...
	'Callback','FourierSeries (''L3Function'')', ...
	'Position',[160 310 80 20], ...
	'String','-x', ...
	'Style','edit');
h_L3Lower = uicontrol(h_Fig, ...
   'backgroundcolor', ed, ...
	'Callback','FourierSeries (''L3Lower'')', ...
	'Position',[260 310 30 20], ...
	'String','1', ...
	'Style','edit');
uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
	'Position',[290 310 30 20], ...
	'String','<x<', ...
	'Style','text');
h_L3Upper = uicontrol(h_Fig, ...
   'backgroundcolor', ed, ...
	'Callback','FourierSeries (''L3Upper'')', ...
	'Position',[320 310 30 20], ...
	'String','3', ...
   'Style','edit');



uicontrol(h_Fig, ...
   'backgroundcolor', col, ...
   'position', [160 265 110 20],...
   'string', 'Degree   k =',...
   'horizontalalignment', 'left',...
   'style','text') ;
h_NumTermsText = uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
   'Position',[230 265 30 20], ...
   'horizontalalignment', 'left',...
   'backgroundcolor', [.6 .8 1], ...
   'String','2', ...
	'Style','text');
h_NumTermsSlider = uicontrol(h_Fig, ...
	'Callback','FourierSeries(''PlotSlider'')', ...
   'Max',10, ...
   'Min', 1, ...
	'Position',[270 265 80 20], ...
	'Style','slider', ...
	'Value',2);




uicontrol(h_Fig,'style','frame',...
   'position',[365 265 1 130]) ;

uicontrol(h_Fig, ...
   'backgroundcolor', col, ...
	'Position',[380 370 100 20], ...
   'String','Fourier Coefficients', ...
   'horizontalalignment', 'left', ...
	'Style','text');
uicontrol(h_Fig, ...
   'backgroundcolor', col, ...
	'Position',[380 345 20 20], ...
   'String','m = ', ...
   'horizontalalignment', 'left', ...
	'Style','text');
h_mText = uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
   'Position',[400 345 20 20], ...
   'horizontalalignment','left',...
	'String','2', ...
	'Style','text');
h_mSlider = uicontrol(h_Fig, ...
	'BackgroundColor',col, ...
	'Callback','FourierSeries (''UpdateFourierCoeff'')', ...
	'Max',15, ...
	'Position',[420 345 130 20], ...
	'Style','slider', ...
	'Value',1);
uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
	'Position',[380 315 40 20], ...
   'String','a_m = ', ...
   'horizontalalignment', 'left', ...
   'Style','text');
h_AmText = uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[420 315 400 20], ...
	'String','', ...
   'Style','text');
uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
	'Position',[380 290 40 20], ...
   'String','b_m =', ...
   'horizontalalignment','left',...
	'Style','text');
h_BmText = uicontrol(h_Fig, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[420 290 400 20], ...
   'String','', ...
   'horizontalalignment', 'left', ...
   'Style','text');
h_SymbolicCheck = uicontrol(h_Fig, ...
	'BackgroundColor',col, ...
	'Callback','FourierSeries(''UpdateFourierCoeff'')', ...
	'Position',[380 265 140 20], ...
	'String','Compute symbolically', ...
	'Style','checkbox', ...
   'Value',0);

if ~havesymbolic
   set(h_SymbolicCheck,'visible','off')
end


uicontrol(h_Fig, ...
	'Callback','FourierSeries(''PlotNow'')', ...
	'Position',[20 370 110 30], ...
	'String','Update Graph');

uicontrol(h_Fig, ...
	'Callback','FourierSeries (''Examples'')', ...
	'Position',[20 335 110 30], ...
   'String','Examples');


uicontrol(h_Fig, ...
	'Callback','FourierSeries (''Help'')', ...
	'Position',[20 300 110 30], ...
   'String','Help');

uicontrol(h_Fig, ...
	'Callback','FourierSeries (''Quit'')', ...
	'Position',[20 265 110 30], ...
   'String','Quit');





r = [h_Fig h_L1Function h_L2Function h_L3Function ...
     h_L1Lower h_L1Upper h_L2Lower h_L2Upper h_L3Lower ...
	  h_L3Upper h_mText h_mSlider h_AmText h_BmText ...
	  h_SymbolicCheck h_NumTermsSlider h_NumTermsText h_Axes ]  ;
