% GRAPH ANIMATOR
%
% Use Graph Animator to explore how the graph of a function
% y=f(x,t) changes as t varies.  For example, if y=f(x,t) is
% the graph of the position of a string at time t then 
% Graph Animator displays an animation of the string as it vibrates.
%    
% Enter the function in the box next to f(x,t)= and the range 
% of x values below, then enter the values over which t will 
% vary.  Click the 'Animate' button to start the animation. 
% To change the animation speed move the slider bar.

% REQUIRES: cleanup.m, errmesg.m
%
% EXAMPLE: Type graphanimator; to run this module.  
%          No arguments are needed. 
%

% AUTHOR: William Stein, February, 1998.
function graphanimator(cmd)

resolution = 250;  	% this is system dependent.
							% make smaller for slower systems.
col=[0.7 0.9 1.0];


h_figs=findobj('tag','tag_graphanimator');

if length(h_figs)>0 
   h_fig=h_figs(1);
   if nargin==0
      % The user is only allowed one instance of this application.
      % If the user asks to create another the currently existing
      % instances is brought to the foreground. 
      figure(h_fig);
      return
   end
end

if nargin==0 
   cmd='Initialize';
end
   
if ~strcmp(cmd, 'Initialize')
   if length(h_figs)==0 
      % No non-initialization commands should be called if 
      % the application has not yet been created.
      disp('graphanimator has not yet been initialized.');
      return
   end
   
   % Retrieve handles and redefine the handle variables.
   h=get(h_fig,'userdata'); 
   h_tmin=h(1); h_tmax=h(2); h_t=h(3); 
   h_f=h(4); h_plot_title=h(5); h_plot=h(6); 
   h_xmin=h(7); h_xmax=h(8); h_speed=h(9);
end


switch cmd
case 'Initialize',
   h_fig=figure('position',[10 10 600 410],...
      'tag','tag_graphanimator',...
      'NumberTitle','off',...
      'name','GRAPH ANIMATOR',...
      'resize','off', ...
      'color', col, ...
      'menu', 'none');
   
   % Create the objects: axes, plot, help button, 
   % function input box, t input box, tmin, tmax, animate button,
   % xmin, xmax.
   
   % Create axes
   h_ax=axes('units','pixels','position',[40 165 520 220]); 
   drawnow;
   zoom on;  
   
   % Create plot
   h_plot=plot([1],[1]);
   
   % Create axes labels
   h_plot_title=get(h_ax,'Title');
   xlabel('x'); ylabel('y');
   
   % Create function input box: f(x)=[ ]
   uicontrol(h_fig,'style','text',...
      'position', [20 92 45 20],...
      'string', 'f(x,t)=',...
      'background', col, ...
      'HorizontalAlignment', 'left');
   h_f=uicontrol(h_fig,'style','edit',...
      'position', [80 95 290 20],...
      'background', 'w',...
      'HorizontalAlignment', 'Left',...
      'callback', 'graphanimator(''UpdatePlot'')',...
      'String','sin(x)+1/3*sin(10*t+5*x)');
 
   % Create xmin and xmax fields.
   h_xmin=uicontrol(h_fig,'style','edit',...
      'position',[80 60 40 20],...
      'background','w',...
      'Callback','graphanimator(''UpdatePlot'')',...
      'String','-10');
   uicontrol(h_fig,'style','text',...
      'position',[20 60 60 20],...
      'background',col,...
      'String','x range', ...
      'HorizontalAlignment', 'left');
   h_xmax=uicontrol(h_fig,'style','edit',...
      'position',[130 60 40 20],...
      'background','w',...
      'Callback','graphanimator(''UpdatePlot'')',...
      'String','10');  
   
   % 'Vary t between [ ] and [ ].'
   h_tmin=uicontrol(h_fig,'style','edit',...
      'position',[80 30 40 20],...
      'background','w',...
      'Callback','graphanimator(''UpdatePlot'')',...
      'String','0');
   uicontrol(h_fig,'style','text',...
      'position',[20 30 60 20],...
      'background',col,...
      'String','t range', ...
      'HorizontalAlignment', 'left');
   h_tmax=uicontrol(h_fig,'style','edit',...
      'position',[130 30 40 20],...
      'background','w',...
      'Callback','graphanimator(''UpdatePlot'')',...
      'String','2');  
     
   % Create 't=' box
   uicontrol(h_fig,'style','text',...
      'position',[190 30 30 20],...
      'string','t = ',...
      'background', col);
   h_t=uicontrol(h_fig,'style','text',...
      'position',[220 30 60 16],...
      'background','w',...
      'HorizontalAlignment','Left',...
      'callback', 'graphanimator(''UpdatePlot'')',...
      'String','1');
   
   
   % Animation speed bar.
   uicontrol(h_fig,'style','text',...
      'position',[300 30 40 20],...
      'HorizontalAlignment','Left',...
      'String','Speed',...
      'background',col);
   h_speed=uicontrol(h_fig,'style','slider',...
      'position',[350 30 80 20],...
      'min',10,'max',120,'value',30);
   
   uicontrol(h_fig,...
	   'position', [470 110 110 30],...
      'String', 'Animate',...
      'Callback', 'graphanimator(''Animate'')');
   
   uicontrol(h_fig,'String','Help',...
      'position',[470 75 110 30],...
      'Callback','graphanimator(''Help'')');
   
   uicontrol(h_fig, 'String', 'Examples',...
      'position',[470 40 110 30],...
      'Callback','graphanimator(''Examples'')') ;
   
   uicontrol(h_fig, 'String', 'Quit',...
      'position',[470 5 110 30],...
      'Callback','graphanimator(''Quit'')') ;
   
   % Save handles in userdata.
   set(h_fig,'userdata',[h_tmin, h_tmax, h_t, h_f, h_plot_title,...
         h_plot, h_xmin, h_xmax, h_speed]);
   
   % Draw the plot.
   graphanimator('UpdatePlot');   
   
case 'UpdatePlot',
   field=0; 
   s_xmin=cleanup(get(h_xmin,'String'));
   eval([s_xmin ';'], 'field=''Minimum x value'';');
   s_xmax=cleanup(get(h_xmax,'String'));
   eval([s_xmax ';'], 'field=''Maximum x value'';');
   s_t=cleanup(get(h_t,'String'));
   eval([s_t ';'], 'field=''Current value of t'';');   
   if field
      errmesg(field,'num'); error(lasterr);
   end      
   
   xmin=str2num(s_xmin);
   xmax=str2num(s_xmax);
   t=str2num(s_t);
   x=linspace(xmin,xmax,resolution);
   
   f=vectorize(cleanup(get(h_f,'string')));
   field=0;
   eval([f ';'], 'field=''f(x,t)'';');
   if field
      errmesg(field,'fun'); error(lasterr);
   end   
   y=eval(f);
   
   set(h_plot,'xdata',x); 
   set(h_plot,'ydata',y);
   set(gca,'XLim',[xmin,xmax]);
%   set(h_plot_title,'String',cleanup(get(h_f,'string')));
   drawnow;
   
case 'Animate',
   field=0; 
   s_tmin=cleanup(get(h_tmin,'String'));
   eval([s_tmin ';'], 'field=''Starting t value'';');
   s_tmax=cleanup(get(h_tmax,'String'));
   eval([s_tmax ';'], 'field=''Ending t value'';');
   s_xmin=cleanup(get(h_xmin,'String')); 
   eval([s_xmin ';'], 'field=''Minimum x value'';');
   s_xmax=cleanup(get(h_xmax,'String'));
   eval([s_xmax ';'], 'field=''Maximum x value'';');
   
   if field
      errmesg(field,'num'); error(lasterr);
   end
   
   tmin=str2num(s_tmin);
   tmax=str2num(s_tmax);
   xmin=str2num(s_xmin); 
   xmax=str2num(s_xmax);
   
   f=vectorize(cleanup(get(h_f,'string')));
   field=0;
   x=0; t=0;
   eval([f ';'], 'field=''f(x,t)'';');
   if field
      errmesg(field,'fun'); error(lasterr);
   end
   
   speed=get(h_speed,'max')-get(h_speed,'value')+get(h_speed,'min');
   
   t_range=linspace(tmin,tmax,speed);
   x=linspace(xmin,xmax,resolution);
   
   set(h_plot,'erasemode','xor');
   set(h_plot,'xdata',x);   
   
   set(h_plot_title,'String','Preparing animation (move mouse to abort)');
   drawnow;
   
   PointerLoc=get(0,'PointerLocation');
   abort=0;
   ylim=get(gca,'YLim');
   clear y
   for i=1:length(t_range)
      t=t_range(i);
      y(i,:)=eval(f);
      set(h_plot,'ydata',y(i,:));
      drawnow;
      if abs(PointerLoc-get(0,'PointerLocation')) * [1 1]' > 20
         abort=1;
         break
      end
   end
   
   y = real(y);
   ylim = [ min(min(y)) max(max(y)) ];
   
   dy = abs([.02 -.02] * ylim');
   xlim = get(gca, 'XLim');
   dx = abs([.02 -.02] * xlim');
   set(gca,'YLim', [ylim(1)-dy ylim(2)+dy]);
   set(gca,'XLim', [xlim(1)-dx xlim(2)+dx]);
   set(h_plot,'erasemode','background');
   
   if abort~=1
      set(h_plot_title,'String','Animating (move mouse to stop)');
      while PointerLoc==get(0,'PointerLocation')
         for i=1:length(t_range)
            set(h_plot,'ydata',y(i,:));
            drawnow;
            if abs(PointerLoc-get(0,'PointerLocation')) * [1 1]' > 20
               break;
            end
         end
      end
   end
   
   set(h_t,'String',num2str(t_range(i)));
   set(h_plot,'erasemode','normal');     
   set(gca,'YLimMod','auto');
   graphanimator('UpdatePlot');
   
case 'Help',
   helpwin ('graphanimator.m')
   
case 'Examples',
   e(1).button = 'Vibrating String' ;
   e(1).text = ['This is the first few terms ' ...
         'in the Fourier expansion of a solution ' ...
         'to the wave equation for a vibrating string. ' ...
         'For more examples like this see Section 10.6 ' ...
         'of Boyce and DiPrima.'] ;
   e(1).data = {'sin(pi*x)*cos(5*pi*t)+.5*sin(2*pi*x)*cos(5*2*pi*t)+sin(3*pi*x)*cos(5*3*pi*t)',...
         '0','1','0','.5'} ;
   e(2).button = 'Traveling Wave' ;
   e(2).text = 'This is a neat looking traveling wave.' ;
   e(2).data = {'sin(x)+1/3*sin(10*t+5*x)', '-10', '10', '0', '2'} ;
      
      
   i = examples ('GRAPH ANIMATOR', e) ;
   if i
      set(h_f,'string',e(i).data{1})
      set(h_xmin,'string',e(i).data{2})
      set(h_xmax,'string',e(i).data{3})
      set(h_tmin,'string',e(i).data{4})
      set(h_tmax,'string',e(i).data{5})
		GraphAnimator UpdatePlot
   end   
   
   
case 'Quit',
   delete (findobj ('tag', 'tag_graphanimator')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
end
