function lagrange(cmd)
% LAGRANGE MULTIPLIERS
% 
% This module helps you examine the geometry behind Lagrange
% multiplier problems in two variables.  On paper, you can
% maximize f(x,y) subject to the constraint that g(x,y) = k by
% solving a system of equations. Here, you can estimate
% solutions by finding which level curves of f are tangent to
% the constraint curve. Enter the function f(x,y) that you
% want to maximize or minimize, the constraint function
% g(x,y), and the domain of x and y values into the
% appropriate boxes. Then, you will the constraint curve
% g(x,y) = 0 plotted in green.  (If you need to plot g(x,y) = k
% for some nonzero constant k, just subtract k from your
% definition of g).
% 
% Click in the window to plot level curves for f.  Your goal
% is to find curves just tangent to the green curve, and to
% interpret this data to solve your original problem.
%
% 1.1/19-July-1998

% Use with Problem 19 (a), Section 12.8, page 806 of Stewart (3rd ed)
%
% HISTORY: 1.0 -- Original by Tom Insel (spring 1998)
%          1.1 -- Updated for Matlab 5, style (summer 1998)
%
% REQUIRES: errmesg.m, cleanup.m
%
% TESTED: MATLAB 5.2 for Macintosh

persistent g_of_xy f_of_xy xlow xhi ylow yhi fig_axes level_text

% colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_oldcurve = [1 .8 0];

if ~nargin,

	main_figures = findobj('tag', 'lagrange_main');
	
	if length(main_figures) == 0
		cmd = 'new';
	else
		figure(main_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')

	fig_han = figure(...
		'name', 'Lagrange Multipliers',...
		'Tag', 'lagrange_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'color', math_bkgnd,...
		'resize', 'off',...
		'WindowButtonDownFcn', 'lagrange(''click'')');

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'FUNCTION',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 370 180 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'f(x,y) =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 350 45 20]);

	f_of_xy = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', 'x*y',...
		'position', [60 350 130 20],...
		'callback', 'lagrange(''draw'')');
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 340 180 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'CONSTRAINT CURVE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 310 180 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'g(x,y) = 0 =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 285 65 20]);

	g_of_xy = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', 'x^2 + y^2 - 1/2',...
		'position', [80 285 110 20],...
		'callback', 'lagrange(''draw'')');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 275 180 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'DOMAIN',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 245 180 20]);
		
	xlow = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-1',...
		'position', [20 220 65 20],...
		'callback', 'lagrange(''draw'')');	

	if strcmp(computer,'MAC2')
		domain_string = ' x ';
	else
		domain_string = '<= x <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [85 220 40 20]);

	xhi = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '1',...
		'position', [125 220 65 20],...
		'callback', 'lagrange(''draw'')');	

	ylow = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-1',...
		'position', [20 195 65 20],...
		'callback', 'lagrange(''draw'')');	

	if strcmp(computer,'MAC2')
		domain_string = ' y ';
	else
		domain_string = '<= y <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [85 195 40 20]);
		
	yhi = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '1',...
		'position', [125 195 65 20],...
		'callback', 'lagrange(''draw'')');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 145 180 1]);

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 185 180 1]);
		
	level_text = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'foregroundcolor', 'r',...
		'position', [10 150 180 20]);

	uicontrol(fig_han,...
		'string', 'Clear Level Curves',...
		'position', [10 100 180 35],...
		'callback', 'lagrange(''draw'');');

	uicontrol(fig_han,...
		'string', 'Examples',...
		'position', [10 55 180 35],...
		'callback', 'lagrange(''examples'');');

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 10 85 35],...
		'callback', 'helpwin lagrange');
		
	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [105 10 85 35],...
		'callback', 'lagrange(''end'');');

	fig_axes = axes(...
		'Position',[.41 .075 .56 .86],...
		'XLimMode', 'auto',...
		'YLimMode', 'auto');

	lagrange('draw');
			
elseif strcmp(cmd, 'draw') | strcmp(cmd, 'click')

	main_figures = findobj('tag', 'lagrange_main');
	if length(main_figures) == 0
		main_figures = lagrange('new');
	end

	main_figure = main_figures(1);

	try
		xlow_n = eval(cleanup(get(xlow,'string')));
		xhi_n = eval(cleanup(get(xhi,'string')));
		ylow_n = eval(cleanup(get(ylow,'string')));
		yhi_n = eval(cleanup(get(yhi,'string')));
	catch
		errmesg('xy-domain', 'num')
	end

	% hi/low make sure in the right order

	if strcmp(cmd, 'draw')

		[x, y] = meshgrid(linspace(xlow_n,xhi_n), linspace(ylow_n,yhi_n));
		
		try
			g = eval(vectorize( cleanup(get(g_of_xy, 'string'))));
		catch
			errmesg('g(x,y)', 'fun')
			return
		end

		c = contour(x, y, g, [0 0], 'b');
			% contour() treats a one-element vector as the number zero & draws nothing,
			% so we specify it twice.
		title('Click to draw a level curve for f.');
		set(level_text, 'string', '');
			
	elseif strcmp(cmd, 'click')
		
		p = get(fig_axes, 'CurrentPoint');
		click_x = p(1, 1);
		click_y = p(1, 2);
		
		x = click_x;
		y = click_y;
		
		try
			f1 = eval(cleanup(get(f_of_xy,'string')));
		catch
			errmesg('f(x,y)', 'fun')
			return
		end
		
		[x, y] = meshgrid(linspace(xlow_n,xhi_n), linspace(ylow_n,yhi_n));
		
		try
			f = eval(vectorize(cleanup(get(f_of_xy,'string'))));
		catch
			errmesg('f(x,y)', 'fun')
			return
		end
		
		hold on
		
		set(findobj(gca, 'tag', 'levelset'),'color', math_oldcurve)
		
		[c,h] = contour(x, y, f, [f1 f1], 'r');
		set(h, 'tag', 'levelset')
		
		hold off

		set(level_text, 'string',...
			['f(' num2str(click_x,2) ',' num2str(click_y,2) ') = ' num2str(f1)]);
	
	end
	
elseif strcmp(cmd, 'examples')

	e(1).button = 'Line/Circle';
	e(1).text = 'Find the point on a line closest to the origin.';
	d(1).data = {'x^2 + y^2', 'x+y+3', '-5', '5', '-5', '5'};
	e(2).button = 'WS 15 Problem 1';
	e(2).text = 'If x + y = 10, what is the maximum value of x*y?';
	d(2).data = {'x*y', 'x + y - 10', '-20', '20', '-20', '20'};
	i = examples('Lagrange Multipliers', e);
	if i
		d = d(i).data ;
		set(f_of_xy, 'string', d{1});
		set(g_of_xy, 'string', d{2});
		set(xlow, 'string', d{3});
		set(xhi, 'string', d{4});
		set(ylow, 'string', d{5});
		set(yhi, 'string', d{6});
		lagrange('draw');
	end
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'lagrange_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
