function result = lineint(cmd)
% 2D VECTOR FIELDS
%
% You can use this module to graph vector fields and integrate along a path.
% 
% Specify the vector field in the form P(x,y) * i + Q(x,y) * j. Then, adjust
% the x-range, y-range, arrow length and arrow density settings to control
% how the vector field is graphed. You can specify the path in the form 
% x(t) * i + y(t) * j by entering x(t), y(t) and the range of t in the
% appropriate boxes.
%
% When you press the INTEGRATE button, the computer will display a numerical
% approximation of the integral along the path in the vector field.
% 
% Select the MARKER button and the cursor will become a crosshairs. Position
% the crosshairs somewhere in the vector field and click. A green cross will
% mark the point you selected.  The coordinates of the point you selected,
% the vector at that point, and the scalar curl and divergence of the vector
% field at that point will appear beside the graph.
%
% 2.0/9-July-1998
			
% History: merges lineint & vector field modules by Alex Gottlieb (1996)
% this is version 2.0 by Tom Insel, Summer 1998.
%
% TODO:
%  Dynamic estimate of integral!
%  problems if lo_ == hi_,
%  intermittent mysterious crashes -- fixed?
%  Should have a flowline feature! Or a different module.

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_inner = [1 .6 .6];
math_outer = [.6 .6 1];

persistent P_input Q_input lo_x hi_x lo_y hi_y x_of_t y_of_t lo_t hi_t
persistent integral_label xy_label pq_label div_label curl_label point_handle arrow_length arrow_density

if ~nargin,

	control_figures = findobj('tag', 'lineint_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd, 'new')

	main_figures = findobj('tag', 'lineint_main');
	if length(main_figures) == 0
		main_figures = lineint('makemain');
	end
	figure(main_figures(1));
	
	lineint('regraph');
		
	if ~havesymbolic
		warndlg({'The Symbolic Toolkit is not installed, so the module',...
			'2D VECTOR FIELDS will not be fully functional.'},...
			'Warning');
	end
	
elseif strcmp(cmd,'makemain')

	fig_han = figure(...
		'name', '2D Vector Fields',...
		'Tag', 'lineint_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'color', math_bkgnd,...
		'renderer', 'zbuffer',... % to prevent flickering with markers
		'resize', 'off');
	
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'position', [10 370 150 20],...
		'string', 'VECTOR FIELD',...
		'backgroundcolor', math_bkgnd);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [10 350 60 20],...
		'string', 'P(x,y) = ',...
		'backgroundcolor', math_bkgnd);

	P_input = uicontrol(fig_han,...
		'style', 'edit',...
		'string','x*y^2',...
		'position', [80 350 110 20],...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);
  
   	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [10 325 60 20],...
		'string', 'Q(x,y) = ',...
		'backgroundcolor', math_bkgnd);

	Q_input = uicontrol(fig_han,...
		'style', 'edit',...
		'string','y^3 - x^3',...
		'position', [80 325 110 20],...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);
		
	% region	
	
   	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [15 300 10 20],...
		'string', 'x:',...
		'backgroundcolor', math_bkgnd);

	lo_x = uicontrol(fig_han,...
		'style', 'edit',...
		'position', [33 300 32 20],...
		'string', '-3',...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);

	hi_x = uicontrol(fig_han,...
		'style', 'edit',...
		'position', [65 300 32 20],...
		'string', '3',...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);
	
   	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [108 300 10 20],...
		'string', 'y:',...
		'backgroundcolor', math_bkgnd);

	lo_y = uicontrol(fig_han,...
		'style', 'edit',...
		'position', [126 300 32 20],...
		'string', '-3',...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);

	hi_y = uicontrol(fig_han,...
		'style', 'edit',...
		'position', [158 300 32 20],...
		'string', '3',...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);
		
	%
	
	uicontrol(fig_han,...
		'style', 'text',...
		'position', [10 275 80 20],...
		'string', 'length',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
	
	arrow_length = uicontrol(fig_han,...
		'style', 'slider',...
		'position', [50 275 140 16],...
		'min', 0.1,...
		'max', 10,...
		'value', 2,...
		'callback', 'lineint(''revector'')');

	uicontrol(fig_han,...
		'style', 'text',...
		'position', [10 250 80 20],...
		'string', 'density',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
	
	arrow_density = uicontrol(fig_han,...
		'style', 'slider',...
		'position', [50 250 140 16],...
		'min', 10,...
		'max', 60,...
		'value', 30,...
		'callback', 'lineint(''revector'')');

	%

	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 245 180 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'position', [10 220 60 20],...
		'string', 'CURVE',...
		'backgroundcolor', math_bkgnd);
	
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [10 200 60 20],...
		'string', 'x(t) = ',...
		'backgroundcolor', math_bkgnd);
		
	x_of_t = uicontrol(fig_han,...
		'style', 'edit',...
		'string','2*sin(t)',...
		'position', [80 200 110 20],...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);
	
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [10 175 60 20],...
		'string', 'y(t) = ',...
		'backgroundcolor', math_bkgnd);	
	
	y_of_t = uicontrol(fig_han,...
		'style', 'edit',...
		'string','2*cos(t)',...
		'position', [80 175 110 20],...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);
	
   	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'position', [10 150 100 20],...
		'string', 'range of t values ',...
		'backgroundcolor', math_bkgnd);

	lo_t = uicontrol(fig_han,...
		'style', 'edit',...
		'position', [120 150 35 20],...
		'string', '-pi',...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);

	hi_t = uicontrol(fig_han,...
		'style', 'edit',...
		'position', [155 150 35 20],...
		'string', 'pi',...
		'callback', 'lineint(''regraph'')',...
		'backgroundcolor', math_field);

	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 145 180 1]);
		
	integral_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 126 180 15]);
	
	xy_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 111 180 15]);
	
	pq_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 96 180 15]);
	
	div_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 81 180 15]);
	
	curl_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 66 180 15]);

	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 65 180 1]);

	%

	uicontrol(fig_han,...
		'string', 'Integrate',...
		'position', [10 38 88 23],...
		'callback', 'lineint(''integrate'');');

	uicontrol(fig_han,...
		'string', 'Marker',...
		'position', [103 38 87 23],...
		'callback', 'lineint(''marker'');');

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 10 88 23],...
		'callback', 'helpwin lineint');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [103 10 87 23],...
		'callback', 'lineint(''end'');');
		
	axes('position', [.41 .075 .56 .86]); % 360 pixels square

	result = fig_han;
			
elseif strcmp(cmd,'regraph') | strcmp(cmd, 'revector')

	main_figures = findobj('tag', 'lineint_main');
	if length(main_figures) == 0
		main_figures = lineint('makemain');
	end
	figure(main_figures(1));
	
	if strcmp(cmd, 'regraph')
		set(xy_label, 'string', []);
		set(pq_label, 'string', []);
		set(div_label, 'string', []);
		set(curl_label, 'string', []);
		set(integral_label, 'string', []);
	elseif point_handle
		% to preserve location of marker
		xp = get(point_handle, 'xdata');
		yp = get(point_handle, 'ydata');
	end

	grid_resolution = round(get(arrow_density, 'value'));
	scale = get(arrow_length, 'value');

	try	
		x0 = str2num(cleanup(get(lo_x, 'string')));
	catch
		errmesg('x0', 'num'), error(lasterr), return
	end

	try	
		x1 = str2num(cleanup(get(hi_x, 'string')));  
	catch
		errmesg('x0', 'num'), error(lasterr), return
	end

	try	
		y0 = str2num(cleanup(get(lo_y, 'string')));
	catch
		errmesg('x0', 'num'), error(lasterr), return
	end

	try	
		y1 = str2num(cleanup(get(hi_y, 'string')));
	catch
		errmesg('x0', 'num'), error(lasterr), return
	end
		
	[x y] = meshgrid( linspace(x0,x1, grid_resolution), linspace(y0,y1, grid_resolution));

	try
		P = eval(vectorize(cleanup(get(P_input, 'string')))) + 0*x;
	catch
		errmesg('P(x,y)', 'fun'), error(lasterr), return
	end

	try
		Q = eval(vectorize(cleanup(get(Q_input, 'string')))) + 0*x;
	catch
		errmesg('Q(x,y)', 'fun'), error(lasterr), return
	end

	hold on
	cla
	axis(1.25*[x0 x1 y0 y1])

	quiver(x,y,P,Q,scale);
	
	if strcmp(cmd, 'revector') & point_handle
		point_handle = plot(xp, yp, 'g+', 'markersize', 15);
	end
	
	hold off
	
	try
		t0 = str2num(cleanup(get(lo_t, 'string')));
		t1 = str2num(cleanup(get(hi_t, 'string')));
	catch
		errmesg('t0 or t1', 'num'), error(lasterr), return
	end
	
	t = linspace( t0, t1, 300);

	try
		xc = eval(vectorize(cleanup(get(x_of_t, 'string')))) + 0*t;
	catch
		errmesg('x(t)', 'fun'), error(lasterr), return
	end

	try
		yc = eval(vectorize(cleanup(get(y_of_t, 'string')))) + 0*t;
	catch
		errmesg('y(t)', 'num'), error(lasterr), return
	end
	
	hold on			
	plot(xc, yc, 'r')		
	hold off
	zoom on

elseif strcmp(cmd, 'integrate')
	
	if ~havesymbolic
		set(integral_label, 'string', 'NEED SYMBOLIC TOOLBOX');
		return
	end

	main_figures = findobj('tag', 'lineint_main');
	if length(main_figures) == 0
		main_figures = lineint('makemain');
	end
	figure(main_figures(1));
		
	x_string = vectorize(cleanup(get(x_of_t, 'string')));
	y_string = vectorize(cleanup(get(y_of_t, 'string')));

	t = linspace(t0, t1, 1000);
	
	try
		x = eval(x_string) + 0*t;
		y = eval(y_string) + 0*t;
	catch
		errmesg('x(t) or y(t)', 'fun');
		error(lasterr);
		return
	end

	dx_dt = eval( vectorize( diff(x_string, 't')));
	dy_dt = eval( vectorize( diff(y_string, 't')));
	
	try
		P = eval(vectorize(cleanup(get(P_input, 'string'))));
		Q = eval(vectorize(cleanup(get(Q_input, 'string'))));
	catch
		errmesg('P or Q', 'fun');
		error(lasterr);
		return
	end
	line_integral = trapz( t, P.*dx_dt + Q.*dy_dt);
		
	set(integral_label, 'string', [ 'Line Integral  ' num2str(line_integral)]);

elseif strcmp(cmd, 'marker')

	main_figures = findobj('tag', 'lineint_main');
	if length(main_figures) == 0
		main_figures = lineint('makemain');
	end
	figure(main_figures(1));
		
	[x y] = ginput(1);
	
	if point_handle
		set(point_handle, 'xdata', x, 'ydata', y);
	else
		hold on
		point_handle = plot( x, y, 'g+', 'markersize', 15);
		hold off
	end
	
	set(xy_label, 'string', ['[x,y] = [' num2str(x) ',' num2str(y) ']']);
	
	P_string = cleanup(get(P_input, 'string'));
	Q_string = cleanup(get(Q_input, 'string'));
	
	try	
		P = eval(P_string);
	catch
		errmesg('P(x,y)', 'fun')
		return
	end

	try	
		Q = eval(Q_string);
	catch
		errmesg('Q(x,y)', 'fun')
		return
	end
	
	set(pq_label, 'string', ['[P,Q] = [' num2str(P) ',' num2str(Q) ']']);
	
	if havesymbolic
		divA = diff( P_string, 'x');
		divB = diff( Q_string, 'y');
		curlA = diff( Q_string, 'x');
		curlB = diff( P_string, 'y');
		set( div_label, 'string', ['div = ' num2str( eval(divA) + eval(divB))])
		set( curl_label, 'string', ['scalar = ' num2str( eval(curlA) - eval(curlB))])
	else
		set( div_label, 'string', 'NEED SYMBOLIC TOOLBOX');
		set( curl_label, 'string', 'NEED SYMBOLIC TOOLBOX');
	end
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'lineint_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end


function r = havesymbolic()
r = ~isempty(findstr(path,'symbolic'));
