function result = revolver(cmd, arg)
% SURFACES OF REVOLUTION
%
% This module allows you to view surfaces of revolution.  You can enter a
% profile function f(x) in the box and you will see both the graph of f and
% this graph revolved around the x-axis.  Click and drag to rotate the
% 3-D graph.
% 
% You can choose one of three modes by selecting the appropriate button:
% 
% Smooth:  Surfaces of revolution
%   Graph the surface of revolution.
% 
% Chunky:  Enclosed volume
%   Approximate the surface by N disks and estimate the enclosed volume.
% 
% Conical:  Arc length and surface area
%   Approximate the surface with N truncated cones and estimate the surface
%   area and the arc length of the profile function.
%
% 2.0/18-July-1998

% HISTORY:
% 1.0 by Julie Mitchell, 9-29-96 / 11-19-97
% 2.0 by Tom Insel, Summer 1998, rewritten for MATLAB 5.2, bug fixes, etc.
%     Area/Length use the same algorithm as Julie's code and Volume estimate
%     is similar (but I think more correct).

persistent radio1 radio2 radio3 graph_mode...
	f_field x0_field x1_field N_field N_label...
	est1_label est2_label axes2d axes3d

% Colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin,

	control_figures = findobj('tag', 'revolver_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')

	fig_han = figure(...
		'name', 'Surfaces of Revolution',...
		'Tag', 'revolver_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 410],...
		'color', math_bkgnd,...
		'resize', 'off',...
		'renderer', 'zbuffer',...
		'backingstore', 'off');
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'MODE:',...
		'position', [10 370 80 20],...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
		
	graph_mode = 1;
		
	radio1 = uicontrol(fig_han,...
		'style', 'radiobutton',...
		'string', 'Smooth: surfaces of revolution',...
		'position', [15 350 175 20],...
		'backgroundcolor', math_bkgnd,...
		'value', 1,...
		'callback', 'revolver(''radio1'')');
		
	radio2 = uicontrol(fig_han,...
		'style', 'radiobutton',...
		'string', 'Chunky: volume by disks',...
		'position', [15 330 175 20],...
		'backgroundcolor', math_bkgnd,...
		'callback', 'revolver(''radio2'')');

	radio3 = uicontrol(fig_han,...
		'style', 'radiobutton',...
		'string', 'Conical: arc length, surface area',...
		'position', [15 310 175 20],...
		'backgroundcolor', math_bkgnd,...
		'callback', 'revolver(''radio3'')');

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'f(x) =',...
		'position', [10 280 35 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
		
	f_field = uicontrol(fig_han,...
		'style', 'edit',...
		'string', 'sin(6*x) + 2',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_field,...
		'callback', 'revolver(''draw'')',...
		'position', [50 280 140 20]);
		
	x0_field = uicontrol(fig_han,...
		'style', 'edit',...
		'string', '0',...
		'backgroundcolor', math_field,...
		'callback', 'revolver(''draw'')',...
		'position', [50 255 50 20]);

	if strcmp(computer,'MAC2')
		domain_string = ' x ';
	else
		domain_string = '<= x <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [100 255 40 20]);

	x1_field = uicontrol(fig_han,...
		'style', 'edit',...
		'string', '1',...
		'backgroundcolor', math_field,...
		'callback', 'revolver(''draw'')',...
		'position', [140 255 50 20]);

	N_label = uicontrol(fig_han,...
		'style', 'text',...
		'string', 'N =',...
		'position', [10 225 35 20],...
		'enable', 'off',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
		
	N_field = uicontrol(fig_han,...
		'style', 'edit',...
		'string', '10',...
		'enable', 'off',...
		'backgroundcolor', math_field,...
		'callback', 'revolver(''draw'')',...
		'position', [50 225 50 20]);
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 215 180 1]);
		
	est1_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 185 180 20]);

	est2_label = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 160 180 20]);
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 55 180 1]);
		
	axes2d = axes('units', 'pixels', 'position', [230 230 360 165]);
	axes3d = axes('units', 'pixels', 'position', [230 15 360 190]);
		
	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 10 85 35],...
		'callback', 'helpwin Revolver');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [105 10 85 35],...
		'callback', 'revolver(''end'');');

	revolver('draw');
	
	if nargout, result = fig_han, end
	
elseif strcmp(cmd, 'draw')

	main_figures = findobj('tag', 'revolver_main');
	if length(main_figures) == 0
		main_figures = revolver('new');
	end
	
	try
		x0 = eval(vectorize(cleanup(get(x0_field, 'string'))));
		x1 = eval(vectorize(cleanup(get(x1_field, 'string'))));
	catch
		errmesg('x-domain', 'num'), error(lasterr)
	end

	% graph_mode: 1 = smooth, 2 = chunky, 3 = conical
	% smooth & conical are essentially the same.
	
	if graph_mode == 1
		N = 40;
	else
		try
			N = floor(eval(cleanup(get(N_field, 'string'))));
		catch
			N = 10;
		end
	
		N = min(max(3,N),40);	
		set(N_field, 'string', num2str(N));
	end

	% setup 2D data & estimates

	if graph_mode == 1 | graph_mode == 3
		
		x = linspace(x0, x1, N+1);
	
		try
			f = eval([vectorize(cleanup(get(f_field, 'string'))) '+0*x']);
			% we add 0*x in case f is a constant
		catch
			errmesg('f', 'fun')
			return
		end
	
		axes(axes2d)
		if graph_mode == 3
			plot(x, f, 'k+', x, f)
		else
			plot(x, f)
		end
	
		if graph_mode == 3 % estimates for Arc Length and Surface Area
			
			delta = (x1-x0)/N;
			fprime = diff(f);
			length_estimate = sum(sqrt(delta.^2+fprime.^2));
			
			F = f;
			F(length(F)) = [];
			area_estimate = sum(2.*pi.*sqrt(delta.^2+fprime.^2).*(abs(F)));
			
		end
				
	elseif graph_mode == 2

		a = linspace(x0, x1, N+1);
		x = a + (x1-x0)/(2*N); 
		x(length(x)) = [];

		try
			f = abs(eval([vectorize(cleanup(get(f_field, 'string'))) '+0*x']));
			% we add 0*x in case f is a constant
		catch
			errmesg('f', 'fun')
			return
		end
	
		v(3*length(a)-6) = 0; % initialize empty arrays (for speed?)
		w = v;
		
		for i=2:length(a)-1
			v(3*i-4:3*i-2) = [a(i) a(i) a(i+1)];
			w(3*i-4:3*i-2) = [f(i-1) f(i) f(i)];
		end
	
		v(1) = x0;
		w(1) = f(1);
		
		axes(axes2d)	
		plot(v, w);
		
		volume_estimate = (x1-x0)*pi*sum(f.^2)/N;

	end

	axes(axes2d)
	dx = abs(x1-x0)/10;
	g0 = min([min(f) 0]);
	g1 = max(f);
	dg = abs(g1-g0)/10;
	hold on
	plot([x0-dx x1+dx], [0 0], 'r')
	hold off
	axis([x0-dx x1+dx g0-dg g1+dg]);
	
	% 3D graph

	axes(axes3d)
	cla
	hold on

	if graph_mode == 2
		for i = 1:N
			r = [f(i); f(i); f(i)];
			theta = (0:20)/20*2*pi;
			y = r * cos(theta);
			z = r * sin(theta);
			x = linspace((x1-x0)*(i-1)/N, (x1-x0)*i/N ,3)' * ones(1,21);
			surfl(x,y,z)
		end
	else
		[z y x] = cylinder(f);
		x = x0 - (x0-x1).*x;
		surfl(x,y,z)
	end

	plot3([x0-2*dx x1+2*dx], [0 0], [0 0], 'r')
	hold off
	view(-20,15)
	
	rotate3d ON
	
	% Rotate3d works on the current figure, not the current axes, so there's
	% now a danger of the user rotating the (2-D) graph of f.  Fixing this
	% would involve rewriting rotate3d, so it's not going to happen.
		
	if graph_mode == 1
		set(est1_label, 'string', '');
		set(est2_label, 'string', '');
	elseif graph_mode == 2
		if strcmp(computer,'MAC2')	
			set(est1_label, 'string', ['Volume ' num2str(volume_estimate,5)]);
		else
			set(est1_label, 'string', ['Volume approx =' num2str(volume_estimate,5)]);
		end					
		set(est2_label, 'string', '');
	elseif graph_mode == 3
		if strcmp(computer,'MAC2')	
			set(est1_label, 'string', ['Arc Length  ' num2str(length_estimate,5)]);
			set(est2_label, 'string', ['Surface Area  ' num2str(area_estimate,5)]);
		else
			set(est1_label, 'string', ['Arc Length approx = ' num2str(length_estimate,5)]);
			set(est2_label, 'string', ['Surface Area approx = ' num2str(area_estimate,5)]);
		end			
	end
		
elseif strcmp(cmd, 'radio1')

	set(radio1, 'value', 1)
	set(radio2, 'value', 0)
	set(radio3, 'value', 0)
	set(N_field, 'enable', 'off')
	set(N_label, 'enable', 'off')
	graph_mode = 1;
	revolver('draw')

elseif strcmp(cmd, 'radio2')

	set(radio1, 'value', 0)
	set(radio2, 'value', 1)
	set(radio3, 'value', 0)
	set(N_field, 'enable', 'on')
	set(N_label, 'enable', 'on')
	graph_mode = 2;
	revolver('draw');

elseif strcmp(cmd, 'radio3')

	set(radio1, 'value', 0)
	set(radio2, 'value', 0)
	set(radio3, 'value', 1)
	set(N_field, 'enable', 'on')
	set(N_label, 'enable', 'on')
	graph_mode = 3;
	revolver('draw');

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'revolver_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
