function RSum(cmd)
% RSUM
%
% This modules displays a finite Riemann sums approximation 
% to a definite integral.  The following four rules 
% are implemented:
%    left, right, midpoint, and trapezoidal
%
% Enter the function in the box to the right of f(x).
% Below f(x), enter the range over which to approximate
% the integral and the number of subdivisions.
% To the right select the approximation rule.
% The approximation to the integral is shown at
% the bottom of the display.   To zoom in, select a
% region with the mouse.  


% REQUIRES: cleanup.m, errmesg.m
%
% EXAMPLE: Type RSum; to run this module.
%          No arguments are needed.
%
% AUTHOR: William Stein, February, 1998.
%         Based on a program of Julie Mitchel, 1996.
h_figs=findobj('Tag','tag_RSum');

if length(h_figs)>0 
   h_fig=h_figs(1);
   if nargin==0
      % When an instance of RSum is already running, the
      % command RSum just brings it to the foreground.
      figure(h_fig);
      return
   end
end

if nargin==0 
   cmd='Initialize';
end

if ~strcmp(cmd, 'Initialize')
   if length(h_figs)==0 
      % No non-initialization commands should be called if 
      % the application has not yet been created.
      disp('Error!: RSum has not yet been initialized.');
      return
   end
   
   % Retrieve handles and redefine the handle variables.
   h=get(h_fig,'Userdata'); 
   h_f=h(1); h_xmin=h(2); h_xmax=h(3); h_N=h(4); h_approx=h(5);
   h_left=h(6); h_midpoint=h(7); h_right=h(8); h_trapezoid=h(9);
   % ...
end

switch cmd
case 'Initialize',	
   h_fig=figure('Position',[10 10 600 410],...
      'Tag','tag_RSum',...
      'NumberTitle','off',...
      'menu', 'none', ...
      'Name','RIEMANN SUMS',...
      'Resize','off');
   
   % Create the objects: axes, help button, 
   % function input box, axes pan. 
   
   % Create axes
   h_ax=axes('Units','pixels','Position',[40 125 520 260]);   
   
   % Create control frames
   col=[0.8 0.9 1.0];
   uicontrol(h_fig,'Style','frame',...
      'Position', [10 10 400 21],...
      'Background', col);
   uicontrol(h_fig,'Style','frame',...
      'Position', [10 30 400 60],...
      'Background', col);
   uicontrol(h_fig,'Style','frame',...
      'Position', [409 10 182 80],...
      'Background', col);
      
   % Create function input box: f(x)=[ ]
   uicontrol(h_fig,'Style','text',...
      'Position', [30 60 30 15],...
      'String', 'f(x)=',...
      'HorizontalAlignment', 'Left',...   
      'Background', col);
   h_f=uicontrol(h_fig,'Style','edit',...
      'Position', [70 60 280 20],...
      'Background', 'w',...
      'HorizontalAlignment', 'Left',...
      'callback', 'RSum(''Update'')',...
      'String','sin(x)');
      % were y=60's
   % 'Integrate from [ ] to [ ].'
   uicontrol(h_fig,'Style','text',...
      'Position',[30 35 100 15],...
      'Background',col,...
      'HorizontalAlignment', 'Left',...   
      'String','Integrate from');
   h_xmin=uicontrol(h_fig,'Style','edit',...
      'Position',[105 35 40 20],...
      'Background','w',...
      'String','-5',...
      'Callback','RSum(''Update'');');
   uicontrol(h_fig,'Style','text',...
      'Position',[145 35 30 15],...
      'Background',col,...
      'String','to');
   h_xmax=uicontrol(h_fig,'Style','edit',...
      'Position',[175 35 40 20],...
      'Background','w',...
      'String','5',...
      'Callback','RSum(''Update'');');
   
   % 'Subdivide into [ ] intervals.'
   uicontrol(h_fig,'Style','text',...
      'Position',[220 35 110 15],...
      'Background',col,...
      'HorizontalAlignment', 'Left',...   
      'String','  subdividing into');
   h_N=uicontrol(h_fig,'Style','edit',...
      'Position',[310 35 40 20],...
      'Background','w',...
      'String','20',...
      'Callback','RSum(''Update'');');
   uicontrol(h_fig,'Style','text',...
      'Position',[355 35 50 15],...
      'Background',col,...
      'HorizontalAlignment', 'Left',...   
      'String','intervals.');
      
   
      % left, midpoint, right, trapezoid exclusive radio buttons
    uicontrol(h_fig,'Style','text',...
      'String','RULE',...
      'Position',[420 72 90 15],...
      'Background',col,...
      'HorizontalAlignment', 'Left');
   h_left=uicontrol(h_fig,'Style','radio',...
      'Position',[420 57 90 15],...
      'Background',col,...
      'String','Left Endpoint',...
      'Value',1,...
      'Callback','RSum(''LeftEndpoint'');');
   h_midpoint=uicontrol(h_fig,'Style','radio',...
      'Position',[420 42 90 15],...
      'Background',col,...
      'String','Midpoint',...
      'Value',0,...
      'Callback','RSum(''Midpoint'');');
   h_right=uicontrol(h_fig,'Style','radio',...
      'Position',[420 27 90 15],...
      'Background',col,...
      'String','Right Endpoint',...
      'Value',0,...
      'Callback','RSum(''RightEndpoint'');');
   h_trapezoid=uicontrol(h_fig,'Style','radio',...
      'Position',[420 12 80 15],...
      'Background',col,...
      'String','Trapezoid',...
      'Value',0,...
      'Callback','RSum(''Trapezoid'');');

   
   % Help button
   uicontrol(h_fig,'String','Help',...
      'Position',[520 50 60 25],...
      'Callback','RSum(''Help'')');
   
   uicontrol(h_fig,'String','Quit',...
      'Position',[520 20 60 25],...
      'Callback','RSum(''Quit'')');

   
   % Approx. of the integral.
   h_approx=uicontrol(h_fig,'Style','text',...
      'Position',[30 12 360 15],...
      'Background',col,...
      'HorizontalAlignment', 'Left');
	   
   % Save handles in userdata.
   set(h_fig,'userdata',[h_f h_xmin h_xmax h_N h_approx ...
      h_left h_midpoint h_right h_trapezoid]);
   
   % Draw the plot.
   RSum('Update');   
   
case 'Update',
   field=0; 
   s_xmin=cleanup(get(h_xmin,'String'));
   eval([s_xmin ';'], 'field=''Minimum x Value'';');
   s_xmax=cleanup(get(h_xmax,'String'));
   eval([s_xmax ';'], 'field=''Maximum x Value'';');
   s_N=cleanup(get(h_N,'String'));
   eval([s_N ';'], 'field=''Number of intervals'';');
   if field
      errmesg(field,'num'); error(lasterr);
   end      
   
   xmin=str2num(s_xmin);
   xmax=str2num(s_xmax);
   N=str2num(s_N);
   if N<3 
      N=3; set(h_N,'String','3');
   end
   if N>500
      N=500; set(h_N,'String','500');
   end
   x=linspace(xmin,xmax,500);
   a=linspace(xmin,xmax,N+1);   
   
   f=vectorize(cleanup(get(h_f,'String')));
   field=0;
   eval([f ';'], 'field=''f(x)'';');
   if field
      errmesg(field,'fun'); error(lasterr);
   end   
   y=eval(f);
   
   hold off;
   plot(x,y);
   hold on;
   xlim=get(gca,'XLim');
   set(gca,'XLim',[xlim(1)-(xlim(2)-xlim(1))/16,...
         xlim(2)+(xlim(2)-xlim(1))/16]);
    
   x=a;
   if get(h_midpoint,'Value')==1
      x=x+(x(2)-x(1))/2;
   elseif get(h_right,'Value')==1
      x=x(2:length(x));
   end
   b=eval(f);
   cp=[1 .2 .2];
   cm=[.2 1 .2];
   
   approx=0;
   if get(h_trapezoid,'Value')==0   % draw non-trapezoidal
      for i=1:(length(a)-1)
         if b(i)>=0 
            fill([a(i) a(i) a(i+1) a(i+1)],...
               [0 b(i) b(i) 0], cp);
         else
            fill([a(i) a(i) a(i+1) a(i+1)],...
               [0 b(i) b(i) 0], cm);
         end
         approx = approx + (a(i+1)-a(i))*b(i);
      end         
   else        % draw trapezoidal
      for i=1:(length(a)-1)
         if b(i)>=0 & b(i+1)>=0
            fill([a(i) a(i) a(i+1) a(i+1)],...
               [0 b(i) b(i+1) 0], cp);
         elseif b(i)<=0 & b(i+1)<=0
            fill([a(i) a(i) a(i+1) a(i+1)],...
               [0 b(i) b(i+1) 0], cm);
         else
            xint=(a(i)*b(i+1)-a(i+1)*b(i))/(b(i+1)-b(i));
            if b(i)>=0
               fill([a(i) a(i) xint], [0 b(i) 0], cp);
               fill([xint a(i+1) a(i+1)], [0 b(i+1) 0], cm);
            else
               fill([a(i) a(i) xint], [0 b(i) 0], cm);
               fill([xint a(i+1) a(i+1)], [0 b(i+1) 0], cp);
            end
         end
         
         approx = approx + (a(i+1)-a(i))*(b(i)+b(i+1))/2;
      end            
   end
   
   plot([xmin xmin],get(gca,'YLim'),'c');
   plot([xmax xmax],get(gca,'YLim'),'c');

   x=linspace(xmin,xmax,500);
   y=eval(f);
   plot(x,y);
   
   zoom on;
   
   set(h_approx,'String',...
      ['This subdivision approximates the integral as ' num2str(approx)]);
   
case 'LeftEndpoint',
   set(h_left,'Value',1); set(h_midpoint,'Value',0); 
   set(h_right,'Value',0); set(h_trapezoid,'Value',0);
   RSum('Update');
   
case 'Midpoint',
   set(h_left,'Value',0); set(h_midpoint,'Value',1); 
   set(h_right,'Value',0); set(h_trapezoid,'Value',0);
   RSum('Update');
   
case 'RightEndpoint',
   set(h_left,'Value',0); set(h_midpoint,'Value',0); 
   set(h_right,'Value',1); set(h_trapezoid,'Value',0);
   RSum('Update');
   
case 'Trapezoid',
   set(h_left,'Value',0); set(h_midpoint,'Value',0); 
   set(h_right,'Value',0); set(h_trapezoid,'Value',1);
   RSum('Update');

case 'Help',
   helpwin ('rsum.m') 
   
case 'Quit', 
   delete (findobj ('tag', 'tag_RSum')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
end

