% SPAN3D
%   
% This is a game which helps to develop intuition about
% the span of two vectors in three dimensional space.
%
% 1. Two blue vectors and a red vector will be drawn
%    in the 3D axis.
% 2. Using the mouse rotate the graph around.  Determine
%    whether or not the red vector is in the plane spanned
%    by the two blue vectors.  You can also click on the
%    Coordinates button for (approximations) of the coordinates
%    of the points.
% 3. Click yes if it is, otherwise click no.
% 4. The program will tell you if you are correct and tally
%    your score.
%
% When you are done, click on Reset to start over.  
%
% William Stein (was@math.berkeley.edu), June 1998.

function Span3d (action)

persistent h_fig h_ax h_question h_yes h_no ...
   h_feedback h_correct h_wrong h_play ...
	correct wrong in_span msg v parallel ...

if nargin == 0
   action = 'init' ;
end

switch action
   
case 'init',
   h_fig = findobj ('Tag', 'Span3d') ;
   if (isempty (h_fig))
      h = Span3dWindow ;
      h_fig = h(1) ;
      h_ax = h(2) ;
      h_question = h(3) ;
      h_yes = h(4) ;
      h_no = h(5) ;
      h_feedback = h(6) ;
      h_correct = h(7) ;
      h_wrong = h(8) ;
      h_play = h(9) ;
   end
   figure (h_fig) 
   rand('state',sum(100*clock))
   Span3d('reset') 
   Span3d('play') 
   
case 'reset',
   delete (get(h_ax, 'children')) 
   set ([h_question h_yes h_no h_feedback h_ax], 'visible', 'off') ;
   set (h_play, 'visible', 'on') ;
   correct = 0 ;
   wrong = 0 ;
   Span3d ('updatescore') 

   
case 'updatescore',
   set (h_correct, 'string', ...
      ['Correct: ' num2str(correct)]) ;
   set (h_wrong, 'string', ...
      ['Wrong: ' num2str(wrong)]) ;
   
case 'help',
   helpwin('Span3d.m') 
   
case 'play',
   too_close = 1 ;
   while too_close 
   	v{1} = [1-2*rand 1-2*rand 1-2*rand] ;
   	v{2} = [1-2*rand 1-2*rand 1-2*rand] ;
   	if rand < .25 
      	in_span = 1 ;
      	v{3} = (1-2*rand) * v{1} + (1-2*rand) * v{2} ;
   	else
         in_span = 0 ;
         v{3} = [1-2*rand 1-2*rand 1-2*rand] ;
         while rank([v{1}' v{2}' v{3}']) < 3
      	   v{3} = [1-2*rand 1-2*rand 1-2*rand] ;
         end
    	end
   
   	for i=1:3
     		v{i} = v{i}/sqrt(sum(v{i}.^2)) ;
      end
      
      if min (abs ([v{1}-v{2} v{1}-v{3} v{2}-v{3}])) > .1 & ...
				(in_span == 1 | det([v{1}; v{2}; v{3}]) > .2)
         too_close = 0 ;
      end
   end
   
   
   delete(get(gca,'children'));
   axis ([-1 1 -1 1 -1 1]) 
   hold on
   for i=1:3
      col = 'b' ;
      if i==3
         col = 'r' ;
      end
      x=v{i}(1) ; y = v{i}(2); z = v{i}(3) ;
      arrow ([0 0 0], [x y z], ...
         'facecolor', col, 'width', 7) ;
   end
      
   grid off
   box off
   set ([h_ax h_question h_yes h_no], 'visible', 'on') ;
   set ([h_feedback h_play], 'visible', 'off') ;
   axis off
   drawnow
   rotate3d on
%   Span3d 'parallel' 
   
case 'coords',
   for i=1:3
		x=v{i}(1) ; y = v{i}(2); z = v{i}(3) ;
	   text (x, y, z, ['(' num2str(x,1) ',' num2str(y,1) ',' num2str(z,1) ')']) 
   end
      
case 'parallel', 
   lineseg(v{1},v{1}+v{2})
   lineseg(v{1},v{1}+v{3})
   lineseg(v{1}+v{3},v{3})
   lineseg(v{3},v{2}+v{3})
   lineseg(v{2}+v{3},v{1}+v{2}+v{3})
   lineseg(v{2},v{2}+v{3})
   lineseg(v{1}+v{2},v{1}+v{2}+v{3})
   lineseg(v{2},v{1}+v{2})
   lineseg(v{1}+v{3},v{1}+v{2}+v{3})
      
case 'hint', 
   Span3d 'coords' 
   drawnow
%   d = det([v{1}; v{2}; v{3}])  ;
%   msgbox(['The determinant is approximately ' num2str(d,'%3.1f') '.'], 'Hint') 
   
   
case 'yes',
   if in_span
      correct = correct + 1 ;
   	msg = 'Good job, it is in the span!' ;   
   else
      wrong = wrong + 1 ;
      msg = 'Wrong, it is not in the span.' ;
   end
   Span3d('done') 
   
   
case 'no',
   if ~in_span
      correct = correct + 1 ;
      msg = 'Correct, it is not in the span!' ;
   else
      wrong = wrong + 1 ;
      msg = 'Sorry, it is in the span.' ;
   end
   Span3d('done')
   
case 'done',
   Span3d('updatescore') 
   set ([h_feedback], 'String', msg, 'visible', 'on') 
   set ([h_question h_yes h_no], 'visible', 'off') 
   set (h_play, 'visible', 'on') 
   
case 'quit', 
   delete (findobj ('tag', 'Span3d')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
   
end


function lineseg (p1, p2) 
line ([p1(1) p2(1)], [p1(2) p2(2)], [p1(3) p2(3)], 'color', 'k') 



function h = Span3dWindow()

col = [.6 .8 1];

h_fig = figure('Color','w', ...
	'MenuBar','none', ...
	'Name','SPAN 3D', ...
   'NumberTitle','off', ...
 	'resize', 'off', ...
	'Position',[20 20 620 410], ...
   'Tag','Span3d');
uicontrol (h_fig, ...
   'position', [0 0 200 420],...
   'style', 'frame',...
   'backgroundcolor', col) ;
h_ax = axes('Parent',h_fig, ...
	'Units','pixels', ...
   'Position',[225 40 370 340]);


h_play = uicontrol('Parent',h_fig, ...
	'Callback','Span3d ''play''', ...
	'Position',[45 320 110 50], ...
	'String','N E X T') ;

h_yes = uicontrol('Parent',h_fig, ...
	'Callback','Span3d ''yes''', ...
	'Position',[30 320 80 30], ...
   'String','YES') ;

h_no = uicontrol('Parent',h_fig, ...
	'Callback','Span3d ''no''', ...
	'Position',[110 320 80 30], ...
	'String','NOT in Span') ;

h_feedback = uicontrol('Parent',h_fig, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[45 270 135 40], ...
	'String','You are correct!', ...
   'Style','text') ;

h_wrong = uicontrol('Parent',h_fig, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[60 190 70 20], ...
	'String','Wrong: 0', ...
	'Style','text') ;

h_correct = uicontrol('Parent',h_fig, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[60 220 70 20], ...
	'String','Correct: 0', ...
   'Style','text') ;

h_question = uicontrol('Parent',h_fig, ...
	'BackgroundColor', col, ...
	'Position',[30 350 150 40], ...
   'String',...
   'Is the red vector in the span of the two blue vectors?', ...
   'Style','text') ;

uicontrol('Parent',h_fig, ...
	'Callback','Span3d ''hint''', ...
	'Position',[45 115 110 30], ...
	'String','Coordinates') ;

uicontrol('Parent',h_fig, ...
	'Callback','Span3d ''help''', ...
	'Position',[45 80 110 30], ...
	'String','Help') ;

uicontrol('Parent',h_fig, ...
	'Callback','Span3d ''reset''', ...
	'Position',[45 45 110 30], ...
   'String','New Game') ;

uicontrol('Parent',h_fig, ...
	'Callback','Span3d(''quit'')', ...
	'Position',[45 10 110 30], ...
	'String','Quit') ;

uicontrol('Parent',h_fig, ...
   'BackgroundColor', col, ...
   'horizontalalignment','left',...
	'Position',[20 250 170 20], ...
	'String','ROTATE GRAPH USING MOUSE.', ...
	'Style','text') ;


h = [h_fig h_ax h_question h_yes h_no ...
      h_feedback h_correct h_wrong h_play] ;

